import torch


def test():
    torch.manual_seed(1)


if __name__ == "__main__":
    print(torch.randint(0, 10, (3, 3)))
    print(torch.randint(0, 10, (3, 3)))
    print("-" * 10)
    test()
    print(torch.randint(0, 10, (3, 3)))
    print(torch.randint(0, 10, (3, 3)))

    # whether manual_seed set seed for gpu
    # ANS: yes
    if torch.cuda.is_available():
        device = torch.device("cuda:0")
        print(torch.randint(0, 10, (3, 3), device=device))



import logging


def get_logger(log_level, logger_name, log_file:str=None):
    # 2023-08-02 19:23:44 [0] "/home/whr/fs_gnn/dist_gcn_train.py", line 844, DEBUG: torch.Size([512, 500])
    # note: 如果是多进程，对logging模块的初始化需要在每个进程中都运行一次
    # 并且，logging模块是线程安全的，但并不是进程安全的
    # 如果要保证进程安全，需要将其它进程的消息汇总到一个进程，然后由同一进程中的某些logger（标识进程）来完成
    log_format = f'%(asctime)s %(name)s "%(pathname)s", line %(lineno)d, %(levelname)s: %(message)s\n'

    assert (
        log_level in logging._nameToLevel.keys()
    ), f"log_level should be one of {logging._nameToLevel.keys()}"

    handler = logging.StreamHandler()
    handler.setLevel(logging._nameToLevel[log_level])
    formatter = logging.Formatter(fmt=log_format)
    handler.setFormatter(formatter)
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging._nameToLevel[log_level])
    logger.addHandler(handler)

    if log_file:
        handler_file = logging.FileHandler(log_file)
        handler_file.setLevel(logging._nameToLevel[log_level])
        handler_file.setFormatter(formatter)
        logger.addHandler(handler_file)

    return logger



from whru.log import get_logger
from whru.utils import path_join, get_abs_path, get_datetime_str

if __name__ == "__main__":
    logger = get_logger("INFO", "test", path_join(get_abs_path(__file__), "../test_data", f"{get_datetime_str()}.log"))
    logger.info("hello world")



from tensorboard.backend.event_processing.event_accumulator import EventAccumulator


def read_tensorboard(path:str):
    """
        step, value, wall_time
    """
    ea = EventAccumulator(path)
    ea.Reload()
    return ea


def test():
    pass


if __name__ == "__main__":
    ea = read_tensorboard("./test_data")
    print(ea.Tags())
    print(ea.Scalars("test_acc"))



import threading
import time
import queue


# 生产者任务
def producer(queue):
    for i in range(5):
        item = f'产品{i}'
        queue.put((item, item))
        print(f'生产者生产了 {item}')
        time.sleep(1)  # 模拟生产耗时


# 消费者任务
def consumer(queue):
    while True:
        item = queue.get()
        print(type(item))
        if item is None:
            break  # 如果接收到None，表示生产结束，消费者停止消费
        print(f'消费者消费了 {item}')
        consume(*item)
        time.sleep(2)  # 模拟消费耗时


def consume(a, b):
    print("consume", a, b)

# 创建一个共享队列
shared_queue = queue.Queue(maxsize=10)

# 创建生产者和消费者线程
producer_thread = threading.Thread(target=producer, args=(shared_queue,))
consumer_thread = threading.Thread(target=consumer, args=(shared_queue,))

# 启动线程
producer_thread.start()
consumer_thread.start()

# 等待线程完成
producer_thread.join()
shared_queue.put(None)  # 生产完毕，放入None通知消费者结束
consumer_thread.join()

print('生产消费过程结束。')



import torch
import random
import numpy
import pickle

from typing import List, Union


class Save:
    def __init__(self) -> None:
        self.np_random_state = None
        self.np_random_state_path = "numpy_random_state.pkl"
        self.init_np_random_state()

    def init_np_random_state(self):
        if os.path.exists(self.np_random_state_path):
            with open(self.np_random_state_path, "rb") as f:
                self.np_random_state = np.load(f, allow_pickle=True)
                np.random.set_state(self.np_random_state)

    def dump_np_random_state(self):
        with open("client_manager.pkl", "wb") as f:
            pickle.dump(self.np_random_state, f)

    def numbers(self):
        print_random()
        # update random state
        self.np_random_state = np.random.get_state()


class Checkpoint:
    def __init__(
        self,
        numpy_flag: bool = True,
        random_flag: bool = True,
        torch_flag: bool = True,
        cuda_devices: Union[List[int], List[str], List[torch.device]] = None,
        RNG_state_path: str = "rng_state.pkl",
    ) -> None:
        """
        check point for state of model, PRNG
        """
        self.numpy_flag = numpy_flag
        self.random_flag = random_flag
        self.torch_flag = torch_flag
        self.cuda_devices = cuda_devices

        self.numpy_random_state = None
        self.random_random_state = None
        self.torch_random_state = [None for _ in range(1 + len(cuda_devices))]

        self.RNG_state_path = RNG_state_path

    def save_RNG_state(self):
        if self.numpy_flag:
            self.save_numpy_RNG_state()
        if self.random_flag:
            self.save_random_RNG_state()
        if self.torch_flag:
            self.save_torch_RNG_state()

    def save_RNG_state_to_file(self):
        data = [
            self.numpy_random_state,
            self.random_random_state,
            [tensor.numpy() for tensor in self.torch_random_state],
        ]
        with open(self.RNG_state_path, "wb") as f:
            pickle.dump(data, f)

    def save_numpy_RNG_state(self):
        # tuple
        self.numpy_random_state = numpy.random.get_state()

    def save_random_RNG_state(self):
        # tuple
        self.random_random_state = random.getstate()

    def save_torch_RNG_state(self):
        self.torch_random_state[0] = torch.random.get_rng_state()
        for index, cuda_device in enumerate(self.cuda_devices):
            self.torch_random_state[index + 1] = torch.cuda.random.get_rng_state(
                cuda_device
            )

    def load(self):
        pass



import inspect
import os
import torch
import random
import numpy

from datetime import datetime


def get_current_function_name() -> str:
    """
    return present function's name
    """
    return inspect.currentframe().f_back.f_code.co_name


def get_cwd():
    print("__file__", __file__)
    print(os.path.abspath(__file__))
    return os.getcwd()


def get_datetime_str():
    """
    2024_04_01_10_01_31
    """
    return datetime.strftime(datetime.now(), "%Y_%m_%d_%H_%M_%S")


def path_join(*args):
    return os.path.join(*args)


def get_abs_path(file):
    """
    Args:
        file (str): __file__

    return abs_path(__file__)
    """
    return os.path.abspath(file)


def set_seed(
    seed: int = 0,
    numpy_flag: bool = True,
    random_flag: bool = True,
    torch_flag: bool = True,
):
    if numpy_flag:
        numpy.random.seed(seed)
    if random_flag:
        random.seed(seed)
    if torch_flag:
        # https://pytorch.org/docs/stable/notes/randomness.html#pytorch-random-number-generator
        # You can use torch.manual_seed() to seed the RNG for all devices (both CPU and CUDA):
        torch.manual_seed(seed)


def read_os_env_var(env_var_name: str, default: str = None):
    env_var = os.environ.get(env_var_name)
    if env_var is None:
        env_var = default
    
    return env_var

def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise ValueError('Boolean value expected. True: yes, true, t, y, 1; False: no, false, f, n, 0.')



from setuptools import setup, find_packages

setup(
    name='whru',
    version='0.4.2',
    description='My custom Python module',
    author='whr819987540',
    author_email='steeliron550@gmail.com',
    packages=find_packages(),
    project_urls={
    "Source": "https://github.com/whr819987540/whru",
    "Bug Tracker": "https://github.com/whr819987540/whru/issues",
    },
    install_requires=[
        "torch>=1.7.0",
        "tensorflow>=2.0.2",
    ],
)